#include <R.h>
#include <Rinternals.h>

#include "gribr.h"

void nfree(void *ptr) {
  if (ptr) {
    free(ptr);
    ptr = NULL;
  }
}

void file_finalizer(SEXP ptr) {
  if(!R_ExternalPtrAddr(ptr)) return;
  R_ClearExternalPtr(ptr);
}

SEXP gribr_is_null_ptr (SEXP gribr_ptr) {
  return ScalarLogical(!R_ExternalPtrAddr(gribr_ptr));
}

void gerror(const char *str, int err) {
  error("gribr: %s\nGRIB ERROR %s", str, codes_get_error_message(err));
}

void gwarn(const char *str, int err) {
  warning("gribr: %s\nGRIB ERROR %s", str, codes_get_error_message(err));
}

void grewind(FILE *file) {
  if (ftell(file) != GRIB_FILE_START) {
    if (fseek(file, GRIB_FILE_START, SEEK_SET)) {
      error("gribr: unable to rewind file");
    }
  }
}

int skip_keys(const char* keyName, int keyType, int err) {
    if (!strcmp(keyName, "zero") ||
        !strcmp(keyName, "one") ||
        !strcmp(keyName, "eight") ||
        !strcmp(keyName, "eleven") ||
        !strcmp(keyName, "false") ||
        !strcmp(keyName, "thousand") ||
        !strcmp(keyName, "file") ||
        !strcmp(keyName, "localDir") ||
        !strcmp(keyName, "7777") ||
        !strcmp(keyName, "oneThousand") ||
        !strcmp(keyName, "hundred") ||
        keyType == CODES_TYPE_BYTES ||
        keyType == CODES_TYPE_LABEL ||
        keyType == CODES_TYPE_MISSING ||
        keyType == CODES_TYPE_SECTION ||
        keyType == CODES_TYPE_UNDEFINED ||
        err) {
      return 1;
    }
    return 0;
}

int choose_keys(const char* keyName) {
    if (!strcmp(keyName, "validityDate") ||
        !strcmp(keyName, "validityTime") ||
        !strcmp(keyName, "edition") ||
        !strcmp(keyName, "shortName") ||
        !strcmp(keyName, "name") ||
        !strcmp(keyName, "units") ||
        !strcmp(keyName, "Nx") ||
        !strcmp(keyName, "Ny") ||
        !strcmp(keyName, "Ni") ||
        !strcmp(keyName, "Nj") ||
        !strcmp(keyName, "dataDate") ||
        !strcmp(keyName, "year") ||
        !strcmp(keyName, "month") ||
        !strcmp(keyName, "day") ||
        !strcmp(keyName, "hour") ||
        !strcmp(keyName, "minute") ||
        !strcmp(keyName, "second") ||
        !strcmp(keyName, "gridType") ||
        !strcmp(keyName, "pl") ||
        !strcmp(keyName, "values") ||
        !strcmp(keyName, "jPointsAreConsecutive") ||
        !strcmp(keyName, "longitudeOfFirstGridPointInDegrees") ||
        !strcmp(keyName, "longitudeOfLastGridPointInDegrees") ||
        !strcmp(keyName, "iDirectionIncrement") ||
        !strcmp(keyName, "packingType") ||
        !strcmp(keyName, "distinctLatitudes") ||
        !strcmp(keyName, "distinctLongitudes") ||
        !strcmp(keyName, "latitudes") ||
        !strcmp(keyName, "longitudes") ||
        !strcmp(keyName, "xDirectionGridLengthInMetres") ||
        !strcmp(keyName, "yDirectionGridLengthInMetres") ||
        !strcmp(keyName, "DxInMetres") ||
        !strcmp(keyName, "DyInMetres") ||
        !strcmp(keyName, "Dx") ||
        !strcmp(keyName, "Dy") ||
        !strcmp(keyName, "grib2divider") ||
        !strcmp(keyName, "Nr") ||
        !strcmp(keyName, "dx") ||
        !strcmp(keyName, "dy") ||
        !strcmp(keyName, "truncateDegrees") ||
        !strcmp(keyName, "shapeOfTheEarth") ||
        !strcmp(keyName, "radius") ||
        !strcmp(keyName, "shapeOfTheEarth") ||
        !strcmp(keyName, "scaleFactorOfMajorAxisOfOblateSpheroidEarth") ||
        !strcmp(keyName, "scaledValueOfEarthMajorAxis") ||
        !strcmp(keyName, "scaleFactorOfRadiusOfSphericalEarth") ||
        !strcmp(keyName, "latitudeWhereDxAndDyAreSpecifiedInDegrees") ||
        !strcmp(keyName, "projectionCentreFlag") ||
        !strcmp(keyName, "orientationOfTheGridInDegrees") ||
        !strcmp(keyName, "LoVInDegrees") ||
        !strcmp(keyName, "LaDInDegrees") ||
        !strcmp(keyName, "Latin1InDegrees") ||
        !strcmp(keyName, "Latin2InDegrees") ||
        !strcmp(keyName, "lon_0") ||
        !strcmp(keyName, "lat_0") ||
        !strcmp(keyName, "lat_1") ||
        !strcmp(keyName, "lat_2") ||
        !strcmp(keyName, "LoV") ||
        !strcmp(keyName, "LoD") ||
        !strcmp(keyName, "Latin1") ||
        !strcmp(keyName, "Latin2") ||
        !strcmp(keyName, "longitudeOfSubSatellitePointInDegrees") ||
        !strcmp(keyName, "latitudeOfSubSatellitePointInDegrees") ||
        !strcmp(keyName, "standardParallel") ||
        !strcmp(keyName, "centralLongitude") ||
        !strcmp(keyName, "longitudeOfFirstGridPoint") ||
        !strcmp(keyName, "longitudeOfLastGridPoint") ||
        !strcmp(keyName, "LaD") ||
        !strcmp(keyName, "Latin") ||
        !strcmp(keyName, "latitudeOfSouthernPoleInDegrees") ||
        !strcmp(keyName, "longitudeOfSouthernPoleInDegrees") ||
        !strcmp(keyName, "angleOfRotationInDegrees"))
{
      return 1;
    }
    return 0;
}

SEXP getListElement(SEXP list, const char *str) {
  int i;
  SEXP elmt = R_NilValue, names = getAttrib(list, R_NamesSymbol);
  for (i = 0; i < length(list); i++) {
    if(strcmp(CHAR(STRING_ELT(names, i)), str) == 0) {
      elmt = VECTOR_ELT(list, i);
      break;
    }
  }
  return elmt;
}

SEXP gribr_is_multi_message(SEXP fileHandle) {
  int err;
  int n_on;
  int n_off;
  codes_handle *h = NULL;
  FILE *file = NULL;

  file = R_ExternalPtrAddr(fileHandle);

  err = codes_count_in_file(DEFAULT_CONTEXT, file, &n_off);
  if (err) {
    gerror("unable to count grib messages", err);
  }

  codes_grib_multi_support_on(DEFAULT_CONTEXT);

  n_on = 0;
  while((h = codes_grib_handle_new_from_file(DEFAULT_CONTEXT, file, &err))) {
    n_on++;
    codes_handle_delete(h);
  }
  if (err) {
    gerror("unable to count grib messages", err);
  }

  /* This rewind is important as neglecting to do so will
   * leave the file handle in a unusable state and cause
   * R to crash */
  if (fseek(file, GRIB_FILE_START, SEEK_SET)) {
    error("gribr: unable to rewind file");
  }

  codes_grib_multi_support_off(DEFAULT_CONTEXT);

  if (n_on != n_off) {
    return ScalarLogical(TRUE);
  }

  return ScalarLogical(FALSE);
}

