SUNSET v2.0.0
=============

Modules: Loading, Units, Anomalies, Calibration, Downscaling, Indices, Skill, Saving, Visualization, Scorecards

New features:
- New module for unit conversion, available for temperature, precipitation and pressure variables.
- New Indices module with the following indices available: NAO, Niño1+2, Niño3, Niño3.4 and Niño4.
- New Scorecards module to create Scorecard visualizations of the computed skill metrics.
- New Downscaling module making use of the in-house CSDownscale package.
- Recipe splitting feature: A recipe can be split into 'atomic recipes' and the same workflow can be easily parallelized for multiple forecast systems, references, variables, start dates and regions.
- Option to load, process, save and plot multiple variables in one atomic recipe.
- Possibility to use Autosubmit 4 as a workflow manager to run recipes in parallel.
- New SUNSET launcher script to split a recipe and run it in parallel directly in the cluster with SLURM or with Autosubmit.
- Option to load tas-tos blend by requesting the 'tas-tos' variable.
- For each module there is the possibility to choose whether or not to save the forecast, hindcast and/or observations.
- New skill metrics MSSS and MSE available.
- New use cases with hands-on tutorials.
- GRIB file loading.

Summary of bugfixes/improvements:

- The names of the module functions have changed to be the same as the name of the module (e.g. load_datasets() has become Loading()). The old names will be deprecated in a future release.
- New datasets and variables added to the seasonal archive.
- The launcher script and the prepare_outputs() function have a new "disable_unique_ID" option, which removes the numerical identifier that is added to the name of the output folder when it is generated.
- The seasonal and decadal loading functions have been renamed and can now sourced and called from the same Loading module.
- Bugfix in the recipe checker: checks are now completed even when the time horizon is not correct.
- The functions have been adapted to the new s2dv_cube structure in CSTools>5.0.0.
- The metric 'corr' (correlation for each ensemble member) has been renamed to 'corr_individual_members'.
- Datasets saved under 'daily' folders in esarchive can now be loaded along with 'daily_mean' datasets.
- Start date information has been added to the plot names to avoid overwriting plots from different start dates.
- In the Visualization section of the recipe, the user can specify which plots to generate, whether they should be in a  single-panel or multi-panel layout, and choose between different projections.
- General improvements to the plots: color scheme, units, titles and subtitles, layout and colorbars.
- Anomalies module can compute anomalies when hindcast and observations do not share the same grid.
- Added module configuration to run SUNSET in the BSC hub.
- Language specification has been added to lubridate functions to ensure they work well in all language locales.

ESS Verification Suite v1.1.0
=============================

Modules: Loading, Anomalies, Calibration, Skill, Saving, Visualization

New features:

- New module for anomaly computation.
- New 'Scorecards' output format (different file names and paths from the default format).
- New 'recipe checker' feature in prepare_outputs(): It runs a series of checks on the recipe to detect potential errors, typos, or missing information.

Summary of fixes/improvements:

- Changed the names of the seasonal systems from the names in /esarchive to the official names in the CDS.
- Fixed a bug in the conversion of precipitation units.
- Fixed a bug related to the consistency between experiment and observation dates for some systems.
- Function parameters have been simplified and uniformized.
- Improvements in the logging functionality for longer messages.
- Improvements to the plots generated by the Visualization module.
- compute_probabilities() now returns the fcst probabilities as well.

ESS Verification Suite v1.0.0
=============================

Modules: Loading, Calibration, Skill, Saving, Visualization

New features:
- New function prepare_outputs(), reads the path to the recipe and prepares a new directory with a unique identifier inside `output_dir`. Returns the recipe with in the form of a list that can be passed to the modules.
- New output structure: The netCDF files and plots are now stored inside the directory mentioned above.
- Log files: stored inside the output directory. Users can specify the logging threshold in the recipe, under Run:Loglevel. The default is `INFO`, which showsm essages of level INFO and higher. They can also indicate if they want the messages displayed in the terminal as well, under Run:Terminal. The default is `TRUE`.

Fixes/improvements:
- The `qmap` calibration method has changed with the new version of CST_QuantileMapping().
- The order of the parameters of the input functions has changed.
- `archive` is no longer a mandatory parameter.
- More informative error message when experiment and reference grids do not match.
- Success messages, loading summaries and warnings are now written to the log file.

