compare_exp_obs_grids <- function(exp, obs) {
  # Check for consistency between exp and obs grid
  if (!isTRUE(all.equal(as.vector(exp$coords$latitude),
                        as.vector(obs$coords$latitude)))) {
    lat_error_msg <- paste("Latitude mismatch between exp and obs.",
                           "Please check the original grids and the",
                           "regrid parameters in your recipe.")
    error(recipe$Run$logger, lat_error_msg)
    exp_lat_msg <- paste0("First exp lat: ", exp$coords$latitude[1],
                           "; Last exp lat: ",
                           exp$coords$latitude[length(exp$coords$latitude)])
    info(recipe$Run$logger, exp_lat_msg)
    obs_lat_msg <- paste0("First obs lat: ", obs$coords$latitude[1],
                          "; Last obs lat: ",
                          obs$coords$latitude[length(obs$coords$latitude)])
    info(recipe$Run$logger, obs_lat_msg)
    stop("exp and obs don't share the same latitudes.")
  }
  if (!isTRUE(all.equal(as.vector(exp$coords$longitude),
                        as.vector(obs$coords$longitude)))) {
    lon_error_msg <- paste("Longitude mismatch between exp and obs.",
                           "Please check the original grids and the",
                           "regrid parameters in your recipe.")
    error(recipe$Run$logger, lon_error_msg)
    exp_lon_msg <- paste0("First exp lon: ", exp$coords$longitude[1],
                           "; Last exp lon: ",
                           exp$coords$longitude[length(exp$coords$longitude)])
    info(recipe$Run$logger, exp_lon_msg)
    obs_lon_msg <- paste0("First obs lon: ", obs$coords$longitude[1],
                          "; Last obs lon: ",
                          obs$coords$longitude[length(obs$coords$longitude)])
    info(recipe$Run$logger, obs_lon_msg)
    stop("exp and obs don't share the same longitudes.")
  }
}
